@@ -22,7 +22,7 @@ struct reg_file_info {
 	struct file_desc	d;
 	RegFileEntry		*rfe;
 	struct file_remap	*remap;
-	bool			size_checked;
+	bool			size_mode_checked;
 	bool			is_dir;
 	char			*path;
 };
